/*
 * bmp085.c
 *
 *  Created on: 08.08.2012
 *      Author: cyblord
 */
#include <inttypes.h>
#include "bmp085.h"

#define SMD500_PARAM_MG      3038        //calibration parameter
#define SMD500_PARAM_MH     -7357        //calibration parameter
#define SMD500_PARAM_MI      3791        //calibration parameter


int16_t bmp085_get_temperature(bmp085_t* p_bmp085, unsigned long ut)
{
	int16_t temperature;
	long x1,x2;
	x1 = (((long) ut - (long) p_bmp085->calibs.ac6) * (long) p_bmp085->calibs.ac5) >> 15;
	x2 = ((long) p_bmp085->calibs.mc << 11) / (x1 + p_bmp085->calibs.md);
	p_bmp085->param_b5 = x1 + x2;
	temperature = ((p_bmp085->param_b5 + 8) >> 4);  // temperature in 0.1C

	return (temperature);
}


uint32_t bmp085_get_pressure(bmp085_t* bmp085, uint32_t up)
{
	long pressure,x1,x2,x3,b3,b6;
	unsigned long b4, b7;

	b6 = bmp085->param_b5 - 4000;
	//*****calculate B3************
	x1 = (b6*b6) >> 12;
	x1 *= bmp085->calibs.b2;
	x1 >>=11;

	x2 = (bmp085->calibs.ac2*b6);
	x2 >>=11;

	x3 = x1 +x2;

	b3 = (((((long)bmp085->calibs.ac1 )*4 + x3) <<bmp085->oversampling_setting) + 2) >> 2;

	//*****calculate B4************
	x1 = (bmp085->calibs.ac3* b6) >> 13;
	x2 = (bmp085->calibs.b1 * ((b6*b6) >> 12) ) >> 16;
	x3 = ((x1 + x2) + 2) >> 2;
	b4 = (bmp085->calibs.ac4 * (unsigned long) (x3 + 32768)) >> 15;

	b7 = ((unsigned long)(up - b3) * (50000>>bmp085->oversampling_setting));
	if (b7 < 0x80000000)
	{
		pressure = (b7 << 1) / b4;
	}
	else
	{
		pressure = (b7 / b4) << 1;
	}

	x1 = pressure >> 8;
	x1 *= x1;
	x1 = (x1 * SMD500_PARAM_MG) >> 16;
	x2 = (pressure * SMD500_PARAM_MH) >> 16;
	pressure += (x1 + x2 + SMD500_PARAM_MI) >> 4;	// pressure in Pa

	return (uint32_t)pressure;
}

uint8_t bmp085_get_cal_param(bmp085_t* bmp085)
{
	uint8_t comres;
	uint8_t data[22];
	comres = bmp085->bus_read(BMP085_I2C_ADDR	, BMP085_PROM_START__ADDR, data, BMP085_PROM_DATA__LEN);

	/*parameters AC1-AC6*/
	bmp085->calibs.ac1 =  (data[0] <<8) | data[1];
	bmp085->calibs.ac2 =  (data[2] <<8) | data[3];
	bmp085->calibs.ac3 =  (data[4] <<8) | data[5];
	bmp085->calibs.ac4 =  (data[6] <<8) | data[7];
	bmp085->calibs.ac5 =  (data[8] <<8) | data[9];
	bmp085->calibs.ac6 = (data[10] <<8) | data[11];

	/*parameters B1,B2*/
	bmp085->calibs.b1 =  (data[12] <<8) | data[13];
	bmp085->calibs.b2 =  (data[14] <<8) | data[15];

	/*parameters MB,MC,MD*/
	bmp085->calibs.mb =  (data[16] <<8) | data[17];
	bmp085->calibs.mc =  (data[18] <<8) | data[19];
	bmp085->calibs.md =  (data[20] <<8) | data[21];
	return comres;

}


uint16_t bmp085_get_ut (bmp085_t* p_bmp085) {
	uint16_t ut;
	uint8_t data[2];
	uint8_t ctrl_reg_data;
	int wait_time;
	int comres;

	ctrl_reg_data = BMP085_T_MEASURE;
	wait_time = BMP085_TEMP_CONVERSION_TIME;
	comres = p_bmp085->bus_write(BMP085_I2C_ADDR, BMP085_CTRL_MEAS_REG, &ctrl_reg_data, 1);


	p_bmp085->delay_msec (wait_time);
	comres += p_bmp085->bus_read(BMP085_I2C_ADDR, BMP085_ADC_OUT_MSB_REG, data, 2);
	ut = (data[0] <<8) | data[1];
	return (ut);
}



uint32_t bmp085_get_up (bmp085_t* p_bmp085) {
	unsigned long up=0;
	unsigned char data[3];
	unsigned char ctrl_reg_data;
	int comres=0;

	ctrl_reg_data = BMP085_P_MEASURE + (p_bmp085->oversampling_setting << 6);
	comres = p_bmp085->bus_write(BMP085_I2C_ADDR, BMP085_CTRL_MEAS_REG, &ctrl_reg_data, 1);
	p_bmp085->delay_msec ( 2 + (3 << (p_bmp085->oversampling_setting) ) );
	comres += p_bmp085->bus_read(BMP085_I2C_ADDR, BMP085_ADC_OUT_MSB_REG, data, 3);
	up = (((unsigned long) data[0] << 16) | ((unsigned long) data[1] << 8) | (unsigned long) data[2]) >> (8-p_bmp085->oversampling_setting);
	return (uint32_t)up;
}
