/*
 * bmp085.h
 *
 *  Created on: 08.08.2012
 *      Author: cyblord
 */

#include <inttypes.h>

#ifndef BMP085_H_
#define BMP085_H_

typedef struct bmp085_calibs {
	int16_t ac1;
	int16_t ac2;
	int16_t ac3;
	uint16_t ac4;
	uint16_t ac5;
	uint16_t ac6;
	int16_t b1;
	int16_t b2;
	int16_t mb;
	int16_t mc;
	int16_t md;
} bmp085_calibs_t;



typedef struct bmp085 {
	bmp085_calibs_t calibs;
	long param_b5;
	uint8_t oversampling_setting;
	uint8_t (*bus_write)( uint8_t,uint8_t,uint8_t *,uint8_t);
	uint8_t (*bus_read)( uint8_t, uint8_t, uint8_t *, uint8_t );
	void (*delay_msec)(uint8_t);
} bmp085_t;

#define BMP085_I2C_ADDR				0xEE
#define BMP085_PROM_START__ADDR		0xaa
#define BMP085_PROM_DATA__LEN		  22

#define BMP085_CHIP_ID_REG			0xD0
#define BMP085_VERSION_REG			0xD1

#define BMP085_CTRL_MEAS_REG		0xF4
#define BMP085_ADC_OUT_MSB_REG		0xF6
#define BMP085_ADC_OUT_LSB_REG		0xF7

#define BMP085_SOFT_RESET_REG		0xE0

#define BMP085_T_MEASURE        	0x2E				// temperature measurent
#define BMP085_P_MEASURE        	0x34				// pressure measurement

#define BMP085_TEMP_CONVERSION_TIME  5				// TO be spec'd by GL or SB



uint8_t bmp085_get_cal_param(bmp085_t* bmp085);
int16_t bmp085_get_temperature(bmp085_t* p_bmp085, unsigned long ut);
uint32_t bmp085_get_pressure(bmp085_t* bmp085, unsigned long up);
uint16_t bmp085_get_ut (bmp085_t* p_bmp085);
uint32_t bmp085_get_up (bmp085_t* p_bmp085);


#endif /* BMP085_H_ */
