/*
 * main.c
 *
 *  Created on: 09.05.2012
 *      Author: cyblord
 */

#include <avr/io.h>
#include <compat/deprecated.h>
#include <util/delay.h>
#include <stdlib.h>
#include <util/crc16.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <util/atomic.h>
#include <math.h>
#include "i2c.h"
#include "mbus.h"
#include "defs.h"
#include "bmp085.h"



/* Echte Spannung in 1/10 Volt : x = (183 y)/1024
 * bei 10k und 2k2 Spannungsteiler
 */

char text[10];

const Config_t default_config={
		7,ADR_DISABLED,4,2,6,ADR_DISABLED,ADR_DISABLED,
		{ALERT_MIN,ALERT_MAX},{ALERT_MIN,ALERT_MAX},{ALERT_MIN,ALERT_MAX},
		5,5
};


bmp085_t bmp085;

const Alert_t ALERT_OFF={ALERT_MIN,ALERT_MAX};

Config_t c;

volatile uint8_t nextMeasurement=0;

long pRef;
long pressure;
int16_t temperature;

int16_t alt_10cm;
int16_t last_alt_10cm=0;
int16_t alt_m;

uint16_t volt;
int16_t vario=0;
int16_t last_vario;

int16_t max_alt=INT16_MIN;
int16_t min_vario=0;
int16_t max_vario=0;

#define VARIO_BUF_LEN		10
#define P_BUF_LEN			40

int16_t vario_buffer[VARIO_BUF_LEN];
uint32_t pressure_buffer[P_BUF_LEN];
uint8_t pressure_buffer_pos=0;

Config_t c_ee EEMEM;
uint8_t config_crc_ee EEMEM;
uint8_t wrong_crc_count_ee;


#define BMP085_BUS_WR_PARAM_ORDER device_addr, register_addr, register_data, write_length
#define BMP085_BUS_RD_PARAM_ORDER device_addr, register_addr, register_data, read_length

uint8_t i2c_write(uint8_t adr, uint8_t reg, uint8_t* data, uint8_t len) {
	uint8_t res=0;
	i2c_start();
	res+=i2c_send(adr);
	res+=i2c_send(reg);
	for (int i=0;i<len;i++) {
		res+=i2c_send(data[i]);
	}
	i2c_stop();
	return res;
}

uint8_t i2c_read(uint8_t adr, uint8_t reg, uint8_t* data, uint8_t len) {
	uint8_t res=0;
	i2c_start();
	res+=i2c_send(adr);
	res+=i2c_send(reg);
	i2c_stop();
	i2c_start();
	res+=i2c_send(adr+1);
	for (int i=0;i<len;i++) {
		data[i]=i2c_receive(i<len-1);
	}
	i2c_stop();
	return res;
}


uint32_t calcAverage(uint32_t *buffer, uint8_t len) {
	uint32_t sum=0;
	for (int i=0;i<len;i++) {
		sum+=buffer[i];
	}
	sum/=len;
	return sum;
}

void bubbleSort(uint32_t *buffer,uint8_t len) {
	uint8_t changed;
	uint32_t temp;
	do {
		changed=0;
		for (uint8_t i=0;i<len-1;i++) {
			if (buffer[i]>buffer[i+1]) { //Switch Items i und i+1
				temp=buffer[i];
				buffer[i]=buffer[i+1];
				buffer[i+1]=temp;
				changed=1;
			}
		}
	}while(changed);
}

uint32_t calcMedian(uint32_t *buffer, uint8_t len) {
	uint8_t index=len/2;
	bubbleSort(buffer,len);
	return buffer[index];
}


void delayms(uint8_t ms) {
	for (int i=0;i<ms;i++) {
		_delay_ms(1);
	}
}

uint8_t calcCRC(Config_t* config) {
	uint8_t crc=0;

	uint8_t* x=(uint8_t*)config;
	for (uint8_t i=0;i<sizeof(Config_t);i++) {
		crc=_crc_ibutton_update(crc,x[i]);
	}
	return crc;
}


void saveConfig(Config_t* config) {
	uint8_t crc=calcCRC(config);
	eeprom_write_block(config,&c_ee,sizeof(Config_t));
	eeprom_write_byte(&config_crc_ee,crc);
}

void loadConfig(Config_t* config) {
	eeprom_read_block(config,&c_ee,sizeof(Config_t));
}

void resetConfig(Config_t* config) {
	*config=default_config;
	saveConfig(config);
}

void sendConfig(Config_t* config) {
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE) {
		uint8_t crc=0;

		mb_send(VERSION);
		crc=_crc_ibutton_update(crc,VERSION);
		uint8_t* x=(uint8_t*)config;
		for (uint8_t i=0;i<sizeof(Config_t);i++) {
			mb_send(x[i]);
			crc=_crc_ibutton_update(crc,x[i]);
		}
		mb_send(crc);
	}
}

#define REC_LEN		(sizeof(Config_t)+2)
void receiveConfig(Config_t* config) {
		uint8_t buffer[REC_LEN];
		for (uint8_t i=0;i<REC_LEN;i++) {
			buffer[i]=mb_receiveBlocked();
		}
		uint8_t version=buffer[0];
		uint8_t rec_crc=buffer[REC_LEN-1];

		uint8_t calced_crc=0;
		for (uint8_t i=0;i<REC_LEN-1;i++) {
			calced_crc=_crc_ibutton_update(calced_crc,buffer[i]);
		}
		_delay_ms(5);

		if (version==VERSION && calced_crc==rec_crc) {
			mb_send(ACK);
			uint8_t *x=(uint8_t*)config;
			for (uint8_t i=0;i<sizeof(Config_t);i++) x[i]=buffer[i+1];
			saveConfig(config);
		}
		else {
			mb_send(NAK);
		}
}

/*
void send_ln(char* s) {
	mb_sendString(s);
	mb_sendString("\r\n");
}

void send_ulong(unsigned long u, int radix, char* t) {
	ultoa(u,t,radix);
	mb_sendString(t);
}
 */
void send_long(long u, int radix, char* t) {
	ltoa(u,t,radix);
	mb_sendString(t);
}



// Hhe in 1/10 m


int16_t height(long p0,long p) {
	double x=(double)p/p0;
	x=(double)-7990*log(x);
	return lround(x*10);
}

// Hhe in 1/10 m

/*
int16_t height2(long p0, long p) {
	double x=p0-p;
	x=x*0.843;
	return lround(x);
}*/



void restoreDefaultSettings() {
	uint8_t confirm=mb_receiveBlocked();
	_delay_ms(10);
	if (confirm==ACK) {
		resetConfig(&c);
		mb_send(ACK);
	}
	else {
		mb_send(NAK);
	}
}

void resetMinMax() {
	max_alt=INT16_MIN;
}

void echo() {
	uint8_t r=mb_receiveBlocked();
	mb_send(r+1);
}

void commandReceived(uint8_t cmd) {
	if (cmd==CMD_SENDDATA) sendConfig(&c);
	else if(cmd==CMD_RECDATA) receiveConfig(&c);
	else if (cmd==CMD_RESET) restoreDefaultSettings();
	else if (cmd==MB_CMD_RESET) resetMinMax();
	else if (cmd==CMD_ECHO) echo();
	else mb_send(NAK);
}



void loadAndCheckConfig() {
	loadConfig(&c);
	uint8_t calced_crc=calcCRC(&c);
	uint8_t stored_crc=eeprom_read_byte(&config_crc_ee);
	if (calced_crc!=stored_crc) {
		resetConfig(&c);
	}
}

uint16_t getVoltage() {

	sbi(ADCSRA,ADSC);
	loop_until_bit_is_clear(ADCSRA,ADSC);

	uint32_t x=ADC;

	x*=ADC_MULTIPLIER;
	x/=ADC_DIVISOR;
	return (uint16_t)x;
}

void init() {
	mb_init();
	mb_setCommandCallback(&commandReceived);
	i2c_init();

#ifdef VSENSE_ON
	ADMUX=VIN_PORT;
	sbi(ADCSRA,ADEN);
	sbi(ADCSRA,ADPS1);
	sbi(ADCSRA,ADPS2);
#endif


	//Timer 1

	sbi(TCCR1,CS10); //clk / 16384
	sbi(TCCR1,CS11);
	sbi(TCCR1,CS12);
	sbi(TCCR1,CS13);
	sbi(TCCR1,CTC1); //CTC Mode
	OCR1C=98; //0,20 Sekunden
	OCR1A=98;
	sbi(TIMSK,OCIE1A);




	bmp085.bus_write=&i2c_write;
	bmp085.bus_read=&i2c_read;
	bmp085.delay_msec=&delayms;
	bmp085.oversampling_setting=OSS;

	bmp085_get_cal_param(&bmp085);
}

void mlink_update(uint8_t adr,int16_t value,uint8_t unit, const Alert_t* alert) {
	if (adr>14) return;
	uint8_t al=MB_NOALERT;
	if (value<alert->min || value>alert->max) al=MB_ALERT;

	mb_update(adr,unit,value,al);

}

int16_t getBMP085Temperature() {
	uint16_t ut=bmp085_get_ut(&bmp085);
	return bmp085_get_temperature(&bmp085,ut);
}

uint32_t getBMP085Pressure() {
	uint32_t up=bmp085_get_up(&bmp085);
	return bmp085_get_pressure(&bmp085,up);
}




ISR(TIM1_COMPA_vect) {
	nextMeasurement=1;
}

int16_t calcVario() {
	int16_t v=vario_buffer[0];
	for (uint8_t i=1;i<VARIO_BUF_LEN-1;i++) {
		v+=vario_buffer[i];
	}
	return v/2;
}

void saveRefPressure() {
	for (int i=0;i<P_BUF_LEN;i++) {
		getBMP085Temperature();
		pressure_buffer[i]=getBMP085Pressure();
		_delay_ms(50);
	}
	pRef=calcMedian(pressure_buffer,P_BUF_LEN);
}

int main() {

	_delay_ms(200);

	init();





	_delay_ms(1000);

	loadAndCheckConfig();



	if (ADR_VALID(c.adr_alt)) mb_novalue(c.adr_alt,MB_ALT);
	if (ADR_VALID(c.adr_maxalt)) mb_novalue(c.adr_maxalt,MB_ALT);
	if (ADR_VALID(c.adr_temp)) mb_novalue(c.adr_temp,MB_TEMP);
	if (ADR_VALID(c.adr_volt)) mb_novalue(c.adr_volt,MB_VOLTAGE);
	if (ADR_VALID(c.adr_vario)) mb_novalue(c.adr_vario,MB_VARIO);
	if (ADR_VALID(c.adr_minvario)) mb_novalue(c.adr_minvario,MB_VARIO);
	if (ADR_VALID(c.adr_maxvario)) mb_novalue(c.adr_maxvario,MB_VARIO);


	sei();

	_delay_ms(4000); //Warm Up


	saveRefPressure();

	uint8_t updateCounter=0;


	int16_t vario_th_up=c.vario_th_up;
	int16_t vario_th_lo=c.vario_th_lo;
	vario_th_lo=0-vario_th_lo;
	uint8_t vario_counter=0;

	getBMP085Temperature();
	pressure=getBMP085Pressure();

	long subPressure;



	while(1) {
		temperature=getBMP085Temperature();

		int i;
		for (i=0;i<P_BUF_LEN;i++) {
			pressure_buffer[i]=getBMP085Pressure();
			if (nextMeasurement) break;
		}
		subPressure=calcMedian(pressure_buffer,i+1);


		while(!nextMeasurement);
		nextMeasurement=0;

		pressure+=subPressure;
		pressure/=2;

		alt_10cm=height(pRef,pressure);

		vario_counter++;
		if (vario_counter==5) {
			vario_counter=0;

			vario=vario+(alt_10cm-last_alt_10cm);
			vario/=2;

			last_alt_10cm=alt_10cm;
		}


		if (vario>=vario_th_lo && vario<=vario_th_up) vario=0;

		if (vario<min_vario) min_vario=vario;
		if (vario>max_vario) max_vario=vario;



		alt_m=lround((double)alt_10cm/10);

		if (alt_m>max_alt) max_alt=alt_m;

		volt=getVoltage();

		updateCounter++;
		if ((updateCounter % 2)==0) {
			mlink_update(c.adr_alt,alt_m,MB_ALT,&c.alert_alt);
			mlink_update(c.adr_maxalt,max_alt,MB_ALT,&ALERT_OFF);
			mlink_update(c.adr_temp,temperature,MB_TEMP,&c.alert_temp);
			mlink_update(c.adr_volt,volt,MB_VOLTAGE,&c.alert_volt);
			mlink_update(c.adr_vario,vario,MB_VARIO,&ALERT_OFF);
			mlink_update(c.adr_minvario,min_vario,MB_VARIO,&ALERT_OFF);
			mlink_update(c.adr_maxvario,max_vario,MB_VARIO,&ALERT_OFF);
		}


	}


	return 0;
}



