/*
 * mbus.c
 *
 *  Created on: 15.03.2012
 *      Author: cyblord
 */

#include <avr/io.h>
#include <compat/deprecated.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "mbus.h"

MB_DATA_T mb_data[16];
volatile uint8_t mb_lastbyte;
volatile uint8_t mb_lastvalid=0;
volatile uint8_t mb_afterRequestByte=0;
void (*mb_commandReceived)(uint8_t);


void mb_timer_freeze() {
	sbi(GTCCR,TSM);
	sbi(GTCCR,PSR0);
}

void mb_timer_restart() {
	TCNT0=0;
	cbi(GTCCR,TSM);
}



uint8_t mb_isStart() {
	return !MB_IS_BUS();
}


uint8_t mb_receiveBlocked() {
	while(!mb_isStart());
	return mb_receive();
}


void mb_send(uint8_t data) {
	MB_BUS_LO();
	_delay_us(MB_BITLEN);

	for (int i=0;i<8;i++) {
		if ((data & 1)>0) MB_BUS_REL();
		else MB_BUS_LO();

		_delay_us(MB_BITLEN);
		data=data>>1;
	}
	MB_BUS_REL();
	_delay_us(MB_BITLEN);

}

void mb_sendString(char* s) {
	while(*s) {
		mb_send(*s);
		s++;
	}
}

uint8_t mb_receive() {
	uint8_t data=0;
	_delay_us(MB_BITLEN);
	_delay_us(MB_BITLEN/2);
	for (int i=0;i<8;i++) {
		data>>=1;
		if (MB_IS_BUS()) data |=128;
		_delay_us(MB_BITLEN);
	}
	return data;
}

void mb_update(uint8_t adr,uint8_t unit,int16_t value,uint8_t alert) {
	uint8_t b1,b2,b3;
	b1=adr;
	b1<<=4;
	b1+=unit;

	if (value!=MB_NOVALUE) {
		value<<=1;
		if (alert==MB_ALERT) value++;
	}

	b2=value;
	b3=(value>>8);
	mb_data[adr].response[0]=b1;
	mb_data[adr].response[1]=b2;
	mb_data[adr].response[2]=b3;
	mb_data[adr].active=1;
}

void mb_novalue(uint8_t adr,uint8_t unit) {
	mb_update(adr,unit,MB_NOVALUE,MB_NOALERT);
}

void mb_disable(uint8_t adr) {
	mb_data[adr].active=0;
}

void mb_init() {

	cbi(MB_PORT,MB_BUS);
	cbi(MB_DDR,MB_BUS);

	sbi(PCMSK,MB_BUS); //Pin Change Interrupt
	sbi(GIMSK,PCIE); //Pin Change Interrupt ein

	//Timer 0
	sbi(TCCR0B,CS02); //clk/256
	OCR0A=50; //CTC bei ca. 1,6 ms
	sbi(TIMSK,OCIE0A); //Compare Match Interrupt ein
	sbi(TIMSK,TOIE0); // Timer 0 Overflow Interrupt ein
	mb_timer_restart();
}

void mb_setCommandCallback(void (*callback)(uint8_t)) {
	mb_commandReceived=callback;
}

void mb_requestDetected(uint8_t adress) { //Anforderung Sensorwerte fr Adresse
	if (adress>MB_MAX_ADRESS) mb_commandReceived(adress);
	else {
		if (mb_data[adress].active==1) {
			mb_send(mb_data[adress].response[0]);
			mb_send(mb_data[adress].response[1]);
			mb_send(mb_data[adress].response[2]);
			sbi(GIFR,INTF0);
		}
	}
}

void mb_startBitDetected() {
	mb_timer_freeze();
	mb_lastbyte=mb_receive();
	mb_lastvalid=(TCNT0>18);
	if (mb_lastvalid) mb_afterRequestByte=1;
	sbi(GIFR,INTF0);
	mb_timer_restart();
}

ISR(TIM0_OVF_vect) { //Timerberlauf -> Timer auf hchstem Wert anhalten
	mb_timer_freeze();
	TCNT0=0xFF;
	mb_afterRequestByte=0;
}

ISR(TIM0_COMPA_vect) { //Timer Compare Match (1,6ms verstrichen seit letztem empfangenem Byte)
	if (mb_lastvalid) {
		mb_requestDetected(mb_lastbyte);
		mb_afterRequestByte=0;
		mb_lastvalid=0;
		mb_timer_restart();
	}

}

ISR(PCINT0_vect) {
	if (!MB_IS_BUS()) mb_startBitDetected(); //Nur auf fallende Flanke des Busses reagieren
}




